#define MAX_SBM_SIZE  30000

#define LDR_HEAD  0
#define LDR_CYL   0
#define LDR_SECT  1

#define LDR_LBA   0

#define KNL_HEAD  0
#define KNL_CYL   0
#define KNL_SECT  2

#define KNL_LBA   1

#define MAJOR_FD        2	/* floppy major */
#define MAJOR_HD        3	/* ide hd major */

#define LDR_SIZE  440
#define SECTOR_SIZE 512
#define MAX_THEME_SIZE  10240

#define SBMT_MAGIC      0x544D4253	/* magic number of
					   Smart Boot Manager theme.
					 */
#define MAX_RECORD_NUM      32
#define SIZE_OF_BOOTRECORD  64

#define SIZE_OF_MBR         446

#ifdef __linux__
#define BACKUP_MAGIC "SBMBAKUP_LINUX\x0a\x00"
#else
#define BACKUP_MAGIC "SBMBAKUP_MSDOS\x0a\x00"
#endif

typedef unsigned long dword;
typedef unsigned short word;
typedef unsigned char byte;

char about[] =
  "\nSmart Boot Manager 3.5 Installer Copyright (C) 2000 Suzhe, Lonius\n"
  "This is free software, you can redistribute it and/or modify it\n"
  "under the terms of the GNU General Public License version 2.\n\n"
  "This program comes with ABSOLUTELY NO WARRANTY!\n\n";

char usage_msg[] =
  " [-t theme] [-d drv_name] [-b backup_file] [-u backup_file] \n\n"
  " -t theme     select the theme to be used, in which the theme could be: \n"
  "                 us = English theme       de = German theme \n"
  "                 hu = Hungarian theme     zh = Chinese theme \n"
  "                 ru = Russian theme       cz = Czech theme \n"
  "                 es = Spanish theme       fr = French theme \n"
  "                 or a filename of user customized theme.\n\n"
#ifdef __linux__
  " -d drv_name  set the drive that you want to install Smart BootManager on.\n"
  "              /dev/fd0 is the first floppy driver, \n"
  "              /dev/hda is the first hard driver.\n\n"
#else
  " -d drv_id    set the drive that you want to install Smart BootManager on.\n"
  "              0   is the first floppy driver,\n"
  "              128 is the first hard driver.\n\n"
#endif
  " -b           backup the data that will be overwrited for future unistallation.\n\n"
  " -u           uninstall the boot loader, should be used alone.\n\n";


struct sbml_header		// the header of smart boot manager
{				// loader.
  dword jmp_cmd;
  dword magic;
  word version;
  byte kernel_sectors;
  byte reserved;
  byte kernel_copies;
  byte kernel_drvid1;
  dword kernel_addr1;
  byte kernel_drvid2;
  dword kernel_addr2;
  byte kernel_drvid3;
  dword kernel_addr3;
}
__attribute__ ((packed));

struct sbmk_header		// the header of smart boot manager
{				// kernel.
  dword jmp_cmd;		// jmp and nop command.
  dword magic;			// magic number.
  word version;			// version.
  word kernel_size;		// the size of kernel code.
  word checksum_size;		// the number of bytes that will
                                // be counted in checksum.
  byte checksum;		// checksum value.
  byte total_sectors;		// the total sectors that kernel
                                // and theme used.
  byte kernel_copies;		// the number of kernel copies.
  byte kernel_drvid1;		// drive id
  dword kernel_addr1;		// LBA address.
  byte kernel_drvid2;
  dword kernel_addr2;
  byte kernel_drvid3;
  dword kernel_addr3;

  byte kernel_flags;
  byte delay_time;
  byte direct_br;
  byte default_record;
  dword root_password;

  byte boot_records[ MAX_RECORD_NUM * SIZE_OF_BOOTRECORD ];
  byte sbml_codes[ SIZE_OF_MBR ];
  byte previous_mbr[ SIZE_OF_MBR ];
}
__attribute__ ((packed));

struct sbmt_header		// the header of smart boot manager
{				// theme.
  dword magic;			// magic number = 'SBMT', 4 bytes.
  word reserved;
  byte lang[6];			// language of this theme, 6 bytes.
  word version;			// version, high byte is major version,
                                // low byte is minor version.
                                // should be equal to the version of
                                // Smart Boot Manager.
  word size;			// size of the theme (bytes).
}
__attribute__ ((packed));
