; asmsyntax=nasm
;
; main.asm
;
; Main programs for Smart Boot Manager
;
; Copyright (C) 2000, Suzhe. See file COPYING for details.
;
%define MAIN
%define HAVE_MAIN_PROG
;%define EMULATE_PROG

%include "macros.h"
%include "ui.h"
%include "hd_io.h"
%include "knl.h"
%include "sbm.h"
%include "main.h"

%define NUM_OF_HOTKEYS      (func_list.entries - func_list.hotkeys)/2
%define NUM_OF_MNUCMDS      (menu_func_list.entries - menu_func_list.hotkeys)/2
%define MAIN_MENU_NITEMS    (func_list.record_mnu_hkeys - func_list.main_mnu_hkeys)/2
%define RECORD_MENU_NITEMS  (func_list.sys_mnu_hkeys - func_list.record_mnu_hkeys)/2
%define SYS_MENU_NITEMS     (func_list.end_of_mnu_hkeys - func_list.sys_mnu_hkeys)/2

        bits 16

%ifdef EMULATE_PROG
        org 0x100
%else
        org 0
%endif

        section .text
        
start_of_kernel:
start_of_checksum_area:

        jmp start
        nop

;!!! PLEASE DON NOT CHANGE THE SIZE AND ORDER OF FOLLOWING DATA !!!

;=============================================================================
;the header of Smart Boot Manager kernel ( 8 bytes )
;=============================================================================
sbmk_magic          dd  SBMK_MAGIC ; magic number = 'SBMK', 4 bytes.
                                   ; it's abbr. of 'Smart Boot Manager Kernel'
sbmk_version        dw  SBMK_VERSION ; version, high byte is major version,
                                   ; low byte is minor version.
kernel_size         dw  (end_of_kernel-start_of_kernel)
checksum_size       dw  (end_of_checksum_area - start_of_checksum_area)
checksum            db  0          ; checksum of the kernel program.

kernel_sectors      db  0          ; the number of sectors that kernel and theme
                                   ; totally used.
kernel_copies       db  0          ; number of kernel copies saved. <= 3
kernel_drvid        db  0          ; the drive id that kernel saved.
kernel_addr         dd  0          ; the abs sector addr that kernel saved.

kernel_drvid2       db  0          ;
kernel_addr2        dd  0          ;

kernel_drvid3       db  0          ;
kernel_addr3        dd  0          ;

;=============================================================================
; MAIN data area
;=============================================================================
kernel_flags        db 0x05                     ; first run signature 
                                                ; and floppy ok signature.
delay_time          db 30                       ; delay time ( seconds )
direct_boot_record  db 0xff                     ; >= MAX_RECORD_NUM means no
                                                ; direct boot.
default_record      db 0xff                     ; the record number will
                                                ; be booted after the
                                                ; delay time is up or ESC
                                                ; key is pressed.
root_password       dd 0

; buffer to store boot records
boot_records        times MAX_RECORD_NUM * SIZE_OF_BOOTRECORD db 0
sbml_codes          times SIZE_OF_MBR db 0
previous_mbr        times SIZE_OF_MBR db 0

bootmenu_style      db 0,0
;=============================================================================

start:

%ifndef EMULATE_PROG                            ; real program goes here.
        cli
        mov ax, cs                              ; initialize the segment
        mov ds, ax                              ; registers and the stack
        mov es, ax                              ; pointer.

        mov ax, STACK_SEG
        mov ss, ax                              ; ss:sp = 0x9000:0x1000
        mov sp, STACK_SIZE                      ;
        sti
%endif

	lea di, [start_of_tmp_data]
	mov cx, end_of_tmp_data - start_of_tmp_data
	xor al, al                              ; clear the temp data area.
	rep stosb                               ;

%ifndef EMULATE_PROG

        mov [use_int13_ext], al
        test byte [kernel_flags], KNLFLAG_NOINT13EXT
        jnz .chk_firstrun
        inc byte [use_int13_ext]

.chk_firstrun:
        test byte [kernel_flags], KNLFLAG_FIRSTRUN
        jz .not_first_run
        
        call init_theme                ; initialize the theme, only one time!
        call init_cmd_menus            ; initialize the command menus
        call init_video                ; initialize the video mode.

        test byte [kernel_flags], KNLFLAG_NOFIRSTSCAN
        jnz .no_first_scan
        call init_boot_records         ; if it's the first time
                                       ; to run this program,
                                       ; call the init_boot_records.

.no_first_scan:
        and byte [kernel_flags], ~ KNLFLAG_FIRSTRUN
        call save_boot_manager
        jc .disk_error
        jmp .show_menu
        
.not_first_run:
        
        call init_video                 ; initialize the video mode.

        mov ah, 0x02                    ; test the keyboard status,
        call bioskey                    ; if ctrl pressed then show
        test al, kbCtrlMask             ; menu directly,
        jnz .show_menu                  ;

        mov al, [direct_boot_record]    ; check if need boot directly.
        cmp al, MAX_RECORD_NUM          ;
        jb .go_direct_boot

        call do_schedule                ; implement the schedule table.

        cmp byte [delay_time], 0
        jnz .show_menu                  ; delay_time = 0, boot the
                                        ; default record directly.
        mov al, [default_record]        ;
        cmp al, MAX_RECORD_NUM
        jb .go_def_boot
        jmp .show_menu
        
.go_direct_boot:
        mov byte [direct_boot_record], 0xff  ; clear the direct boot sig.
        call save_boot_manager
        jc .disk_error

.go_def_boot:
        mov bl, SIZE_OF_BOOTRECORD
        mul bl                          ; get the pointer to the
        lea si, [boot_records]          ; direct/default boot record.
	add si, ax

        call check_bootrecord           ; check if it's valid
        jc .invalid_record

        call boot_the_record            ; boot it.
        
        or al, al
        jz .no_system

.disk_error:
        call show_disk_error
        jmp .show_menu

.no_system:
        mov si, [str_idx.no_system]
        jmp .report_error

.invalid_record:
        mov si, [str_idx.invalid_record]
.report_error:
        call error_box

%else
        call init_cmd_menus                     ; initialize the command menus
        call init_video                         ; here is the code for
        call init_boot_records                  ; emulate program.
        call do_schedule
%endif

.show_menu:
	call init_bootmenu_width
.not_hideflags:
        
        mov al, [delay_time]                    ;
        cmp al, 255                             ;
        jae .not_count_time                     ;
                                                ; if delay_time = 255
        mov [time_count], al                    ; then do not count time.
        xor al, al                              ;
.not_count_time:                                ;
        mov [key_pressed], al                   ;
        
        call init_good_record_list

        mov bl, [default_record]
        lea si, [good_record_list]
        mov cl, [good_record_number]
        xor ch, ch
        xor bh, bh
        
        cld
.loop_search_def:
        lodsb
        cmp al, bl
        je .found_def
        inc bh
        loop .loop_search_def
        jmp .go_ahead

.found_def:
        mov [focus_record], bh
        cmp bh, [size.bootmenu_height]
        jb .go_ahead
        inc bh
        sub bh, [size.bootmenu_height]
        mov [first_visible_rec], bh

.go_ahead:
        call draw_screen

;=============================================================================
;msg_dispatcher ---- the main loop to handle user command
;=============================================================================
msg_dispatcher:
.loop_dispatch:
        call draw_bootmenu

        call get_key                            ; get a key

        lea si, [func_list.hotkeys]             ; search in hotkey list
        mov dx, ax                              ;
        mov cx, NUM_OF_HOTKEYS                  ;
        xor bx, bx                              ;
        cld
.loop_search:                                   ;
        lodsw                                   ;
        cmp dx, ax                              ;
        je .found_it                            ; found, go get func entry
        inc  bx                                 ;
        loop .loop_search                       ;
        jmp .loop_dispatch                      ; not found, wait more

.found_it:
        shl bx, 1
        call word [bx + func_list.entries]      ; call the function
        jmp .loop_dispatch

.halt:  jmp .halt


;=============================================================================
; do_nothing ---- just return!
;=============================================================================
do_nothing:
        ret

;=============================================================================
;include area
;=============================================================================

%include "main-cmds.asm"
%include "main-utils.asm"
%include "ui.asm"
%include "utils.asm"
%include "knl.asm"
%include "hd_io.asm"

;=============================================================================
; data area
;=============================================================================

func_list:
.hotkeys:
        dw  kbUp
        dw  kbEnhUp
        dw  kbCtrlUp
        dw  kbEnhCtrlUp
        dw  kbDown
        dw  kbEnhDown
        dw  kbCtrlDown
        dw  kbEnhCtrlDown

        dw  kbEnhEnter
        dw  kbEsc
        dw  kbTab

.main_mnu_hkeys:
        dw  kbF1
        dw  kbCtrlF1
        dw  kbF2
        dw  kbEnter
        dw  0
        dw  0
        dw  0
        dw  0
        dw  0
        dw  kbCtrlQ
        dw  kbCtrlF12

.record_mnu_hkeys:
        dw  kbSlash
        dw  kbF3
        dw  kbF9
        dw  kbCtrlS
        dw  kbCtrlK
        dw  0
        dw  kbF4
        dw  kbF5
        dw  kbF6
        dw  kbF7
        dw  kbCtrlX
        dw  0
        dw  kbCtrlD
        dw  kbCtrlP
        dw  kbCtrlU
        dw  kbCtrlN

.sys_mnu_hkeys:
        dw  kbF10
        dw  kbCtrlF10
        dw  kbAltF10
        dw  0
        dw  kbF8
        dw  kbShiftF8
        dw  kbCtrlT
        dw  kbCtrlF
        dw  kbCtrlL
        dw  0
        dw  0
        dw  kbCtrlI
        dw  kbCtrlH
        dw  0
        dw  0
        dw  0

.end_of_mnu_hkeys:
        dw  kbGrayPlus
        dw  kbDel
        dw  kbEnhDel
        dw  kbHome
        dw  kbEnhHome
        dw  kbPgDn
        dw  kbEnhPgDn
        dw  kbEnd
        dw  kbEnhEnd
        dw  kbQuestion

;=======================================================
.entries:
        dw  up_cursor                       ; kbUp
        dw  up_cursor                       ; kbUp
        dw  up_cursor                       ; kbUp
        dw  up_cursor                       ; kbUp
        dw  down_cursor                     ; dbDown
        dw  down_cursor                     ; dbDown
        dw  down_cursor                     ; dbDown
        dw  down_cursor                     ; dbDown
        dw  boot_it                         ; kbEnter
        dw  boot_default                    ; kbEsc
        dw  show_main_menu                  ; kbTab

.main_mnu_cmds:
        dw  show_help                       ; kbF1
        dw  show_about                      ; kbCtrlF1
        dw  save_changes                    ; kbF2
        dw  boot_it                         ; kbEnter
        dw  boot_prev_mbr                   ; nokey
        dw  do_nothing                      ; nokey
        dw  show_record_menu                ; nokey
        dw  show_sys_menu                   ; nokey
        dw  do_nothing                      ; nokey
        dw  return_to_bios                  ; kbCtrlQ
        dw  do_power_off                    ; kbCtrlF12

.record_mnu_cmds:
        dw  show_record_info                ; kbSlash
        dw  change_name                     ; kbF3
        dw  change_record_passwd            ; kbF9
        dw  toggle_schedule                 ; kbCtrlS
        dw  toggle_keystrokes               ; kbCtrlK
        dw  do_nothing                      ; nokey
        dw  mark_active                     ; kbF4
        dw  toggle_hidden                   ; kbF5
        dw  toggle_auto_active              ; kbF6
        dw  toggle_auto_hide                ; kbF7
        dw  toggle_swapid                   ; kbCtrlX
        dw  do_nothing                      ; nokey
        dw  delete_record                   ; kbCtrlD
        dw  dup_bootrecord                  ; kbCtrlP
        dw  move_record_up                  ; kbCtrlU
        dw  move_record_down                ; kbCtrlN

.sys_mnu_cmds:
        dw  change_root_passwd              ; kbF10
        dw  login_as_root                   ; kbCtrlF10
        dw  change_security_mode            ; kbAltF10
        dw  do_nothing                      ; nokey
        dw  set_default_record              ; kbF8
        dw  unset_default_record            ; kbShiftF8
        dw  set_delay_time                  ; kbCtrlT
        dw  change_bootmenu_style           ; kbCtrlF
        dw  toggle_rem_last                 ; kbCtrlL
        dw  toggle_int13ext                 ; nokey
        dw  do_nothing                      ; nokey
        dw  rescan_all_drives               ; kbCtrlI
        dw  rescan_fixed_drives             ; kbCtrlH
        dw  do_nothing                      ; nokey
        dw  install_sbm                     ; nokey
        dw  uninstall_sbm                   ; nokey

.end_of_mnu_cmds:
        dw  change_video_mode               ; kbGrayPlus
        dw  move_main_win_left              ; kbDel
        dw  move_main_win_left              ; kbDel
        dw  move_main_win_up                ; kbHome
        dw  move_main_win_up                ; kbHome
        dw  move_main_win_right             ; kbPgDn
        dw  move_main_win_right             ; kbPgDn
        dw  move_main_win_down              ; kbEnd
        dw  move_main_win_down              ; kbEnd
        dw  show_record_info                ; kbQuestion

;==========================================================
menu_func_list:
.hotkeys:
        dw  kbDel
        dw  kbEnhDel
        dw  kbHome
        dw  kbEnhHome
        dw  kbPgDn
        dw  kbEnhPgDn
        dw  kbEnd
        dw  kbEnhEnd

        dw  kbCtrlUp
        dw  kbEnhCtrlUp
        dw  kbCtrlDown
        dw  kbEnhCtrlDown

        dw  kbCtrlHome
        dw  kbEnhCtrlHome
        dw  kbCtrlEnd
        dw  kbEnhCtrlEnd
        dw  kbCtrlDel
        dw  kbEnhCtrlDel
        dw  kbCtrlPgDn
        dw  kbEnhCtrlPgDn

.entries:
        dw  move_menu_left
        dw  move_menu_left
        dw  move_menu_up
        dw  move_menu_up
        dw  move_menu_right
        dw  move_menu_right
        dw  move_menu_down
        dw  move_menu_down
        dw  up_cur_in_menu
        dw  up_cur_in_menu
        dw  down_cur_in_menu
        dw  down_cur_in_menu

        dw  move_main_win_up
        dw  move_main_win_up
        dw  move_main_win_down
        dw  move_main_win_down
        dw  move_main_win_left
        dw  move_main_win_left
        dw  move_main_win_right
        dw  move_main_win_right


;========================================================
;command menus
;========================================================
main_menu  istruc struc_menu
        at struc_menu.title,       dw 0
        at struc_menu.n_items,     dw MAIN_MENU_NITEMS
        at struc_menu.cur_item,    dw 0
        at struc_menu.win_pos,     dw 0
        at struc_menu.menu_size,   dw 0
        at struc_menu.win_size,    dw 0
        at struc_menu.win_attr,    dw 0
        at struc_menu.norm_attr,   dw 0
        at struc_menu.focus_attr,  dw 0
        at struc_menu.str_tbl,     dw str_idx.main_menu_strings
        at struc_menu.cmd_tbl,     dw func_list.main_mnu_cmds
        at struc_menu.hkey_tbl,    dw func_list.main_mnu_hkeys
        at struc_menu.mnu_cmd_num, dw NUM_OF_MNUCMDS
        at struc_menu.mnu_cmd_tbl, dw menu_func_list.entries
        at struc_menu.mnu_hkey_tbl,dw menu_func_list.hotkeys
        at struc_menu.post_req_cmd,dw 0
        iend

record_menu  istruc struc_menu
        at struc_menu.title,       dw 0
        at struc_menu.n_items,     dw RECORD_MENU_NITEMS
        at struc_menu.cur_item,    dw 0
        at struc_menu.win_pos,     dw 0
        at struc_menu.menu_size,   dw 0
        at struc_menu.win_size,    dw 0
        at struc_menu.win_attr,    dw 0
        at struc_menu.norm_attr,   dw 0
        at struc_menu.focus_attr,  dw 0
        at struc_menu.str_tbl,     dw str_idx.record_menu_strings
        at struc_menu.cmd_tbl,     dw func_list.record_mnu_cmds
        at struc_menu.hkey_tbl,    dw func_list.record_mnu_hkeys
        at struc_menu.mnu_cmd_num, dw NUM_OF_MNUCMDS
        at struc_menu.mnu_cmd_tbl, dw menu_func_list.entries
        at struc_menu.mnu_hkey_tbl,dw menu_func_list.hotkeys
        at struc_menu.post_req_cmd,dw draw_screen
        iend

sys_menu  istruc struc_menu
        at struc_menu.title,       dw 0
        at struc_menu.n_items,     dw SYS_MENU_NITEMS
        at struc_menu.cur_item,    dw 0
        at struc_menu.win_pos,     dw 0
        at struc_menu.menu_size,   dw 0
        at struc_menu.win_size,    dw 0
        at struc_menu.win_attr,    dw 0
        at struc_menu.norm_attr,   dw 0
        at struc_menu.focus_attr,  dw 0
        at struc_menu.str_tbl,     dw str_idx.sys_menu_strings
        at struc_menu.cmd_tbl,     dw func_list.sys_mnu_cmds
        at struc_menu.hkey_tbl,    dw func_list.sys_mnu_hkeys
        at struc_menu.mnu_cmd_num, dw NUM_OF_MNUCMDS
        at struc_menu.mnu_cmd_tbl, dw menu_func_list.entries
        at struc_menu.mnu_hkey_tbl,dw menu_func_list.hotkeys
        at struc_menu.post_req_cmd,dw 0
        iend


;END OF KERNEL
        dw BR_GOOD_FLAG
end_of_kernel:

;=============================================================================
;theme data
;=============================================================================
theme_start:

%ifdef THEME_ZH
%include "themes/theme-zh.asm"
%elifdef THEME_DE
%include "themes/theme-de.asm"
%elifdef THEME_HU
%include "themes/theme-hu.asm"
%elifdef THEME_RU
%include "themes/theme-ru.asm"
%elifdef THEME_CZ
%include "themes/theme-cz.asm"
%elifdef THEME_ES
%include "themes/theme-es.asm"
%elifdef THEME_FR
%include "themes/theme-fr.asm"
%else
%include "themes/theme-us.asm"
%endif

;=============================================================================
; temp data area
;=============================================================================
SIZE_OF_SBMK equ ($-$$)
	section .bss

%ifndef EMULATE_PROG
        resb MAX_SBM_SIZE - SIZE_OF_SBMK   ; skip enough space for theme.
%endif
 
start_of_tmp_data:
%include "tempdata.asm"
end_of_tmp_data:

